package javax.security.auth.kerberos;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.Vector;
import java.io.ObjectOutputStream;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.ArrayList;
import java.io.ObjectStreamField;
import java.security.Permission;
import java.util.List;
import java.io.Serializable;
import java.security.PermissionCollection;

final class KrbDelegationPermissionCollection extends PermissionCollection implements Serializable
{
    private transient List<Permission> perms;
    private static final long serialVersionUID = -3383936936589966948L;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public KrbDelegationPermissionCollection() {
        this.perms = new ArrayList();
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof DelegationPermission)) {
            return false;
        }
        synchronized (this) {
            Iterator<Permission> iterator = this.perms.iterator();
            while (iterator.hasNext()) {
                if (((Permission)iterator.next()).implies(permission)) {
                    return true;
                }
                continue;
            }
        }
        return false;
    }
    
    @Override
    public void add(Permission obj) {
        if (!(obj instanceof DelegationPermission)) {
            throw new IllegalArgumentException("invalid permission: " + obj);
        }
        if (this.isReadOnly()) {
            throw new SecurityException("attempt to add a Permission to a readonly PermissionCollection");
        }
        synchronized (this) {
            this.perms.add(0, obj);
        }
        return;
    }
    
    @Override
    public Enumeration<Permission> elements() {
        synchronized (this) {
            return Collections.enumeration(this.perms);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Vector vector = new Vector(this.perms.size());
        synchronized (this) {
            vector.addAll(this.perms);
        }
        objectOutputStream.putFields().put("permissions", vector);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        Vector<Object> vector = (Vector)objectInputStream.readFields().get("permissions", null);
        (this.perms = new ArrayList(vector.size())).addAll(vector);
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("permissions", Vector.class) };
    }
}
